/**
 * JASMINe Deploy ME [Managed Element]
 * Copyright (C) 2008-2012 Bull S.A.S.
 * Copyright (C) 2008-2012 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DeployMEPropertiesManager.java 9977 2012-04-13 08:10:00Z cazauxj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.deployme.api;

import java.util.List;
import java.util.ResourceBundle;

/**
 * Reads and manages the properties files
 * 
 * @author Remy Bresson
 */
public class DeployMEPropertiesManager {

    /**
     * The associated resource bundle to the properties file
     */
    private static ResourceBundle properties;

    /**
     * Properties file path (in properties path format)
     */
    private static String DEPLOYME_PROPERTIES_NAME = "properties.deployme";

    /**
     * Key for the deployme 1.x XSD topology path in the properties file
     */
    private static String DEPLOYME_1_KEY_XSD_TOPOLOGY = "xsdTopologyV1";

    /**
     * Key for the deployme 1.x's topology XML namespace in the properties file
     */
    private static String DEPLOYME_1_KEY_XMLNS_TOPOLOGY = "xmlnsTopologyV1";

    /**
     * Key for the topology XML namespace in the properties file
     */
    private static String DEPLOYME_1_KEY_SCHEMA_LOCATION_TOPOLOGY = "schemaLocationTopologyV1";

    /**
     * Key for the deployme 2.x XSD topology path in the properties file
     */
    private static String DEPLOYME_2_KEY_XSD_TOPOLOGY = "xsdTopologyV2";

    /**
     * Key for the deployme 2.x's topology XML namespace in the properties file
     */
    private static String DEPLOYME_2_KEY_XMLNS_TOPOLOGY = "xmlnsTopologyV2";

    /**
     * Key for the topology XML namespace in the properties file
     */
    private static String DEPLOYME_2_KEY_SCHEMA_LOCATION_TOPOLOGY = "schemaLocationTopologyV2";

    static {
        properties = ResourceBundle.getBundle(DeployMEPropertiesManager.DEPLOYME_PROPERTIES_NAME);
    }

    /**
     * Return the topology schema path
     * @param deploymeVersion {@link DeploymeVersion}
     * @return the topology schema path
     */
    public static String getXsdTopologyPath(final DeploymeVersion deploymeVersion) {
        if (DeploymeVersion.DEPLOYME_1.equals(deploymeVersion)) {
            return properties.getString(DeployMEPropertiesManager.DEPLOYME_1_KEY_XSD_TOPOLOGY);
        } else if (DeploymeVersion.DEPLOYME_2.equals(deploymeVersion)) {
            return properties.getString(DeployMEPropertiesManager.DEPLOYME_2_KEY_XSD_TOPOLOGY);
        }
        return null;
    }

    /**
     * Return the topology XML namespace
     * @param deploymeVersion {@link DeploymeVersion}
     * @return the topology XML namespace
     */
    public static String getTopologyXMLNS(final DeploymeVersion deploymeVersion) {
        if (DeploymeVersion.DEPLOYME_1.equals(deploymeVersion)) {
            return properties.getString(DeployMEPropertiesManager.DEPLOYME_1_KEY_XMLNS_TOPOLOGY);
        } else if (DeploymeVersion.DEPLOYME_2.equals(deploymeVersion)) {
            return properties.getString(DeployMEPropertiesManager.DEPLOYME_2_KEY_XMLNS_TOPOLOGY);
        }
        return null;
    }

    /**
     * Return the topology schema location
     * @param deploymeVersion {@link DeploymeVersion}
     * @return the topology schema location
     */
    public static String getTopologySchemaLocation(final DeploymeVersion deploymeVersion) {
        if (DeploymeVersion.DEPLOYME_1.equals(deploymeVersion)) {
            return properties.getString(DeployMEPropertiesManager.DEPLOYME_1_KEY_SCHEMA_LOCATION_TOPOLOGY);
        } else if (DeploymeVersion.DEPLOYME_2.equals(deploymeVersion)) {
            return properties.getString(DeployMEPropertiesManager.DEPLOYME_2_KEY_SCHEMA_LOCATION_TOPOLOGY);
        }
        return null;
    }

    /**
     * @param namespace A XML namesapce
     * @return the correct deployme version according to the given namespace
     */
    public static DeploymeVersion getDeploymeVersion(final String namespace) {
        if (properties.getString(DeployMEPropertiesManager.DEPLOYME_1_KEY_XMLNS_TOPOLOGY).equals(namespace)) {
            return DeploymeVersion.DEPLOYME_1;           
        } else if (properties.getString(DeployMEPropertiesManager.DEPLOYME_2_KEY_XMLNS_TOPOLOGY).equals(namespace)) {
            return DeploymeVersion.DEPLOYME_2;
        } else {
            return null;
        }
    }

    /**
     * @param namespaces A list of namespaces
     * @return the correct deployme version according to the given list of namespaces
     */
    public static DeploymeVersion getDeploymeVersion(final List<String> namespaces) {
        DeploymeVersion deploymeVersion = null;
        for (String namespace: namespaces) {
            deploymeVersion = getDeploymeVersion(namespace);
            if (deploymeVersion != null) {
                return deploymeVersion;
            }
        }
        return null;
    }
}
